import os
from typing import NamedTuple, List, Union
import shutil
from tqdm import tqdm
import logging

from photoarchiver.helpers.path import get_destination_path, get_deduplicated_destination_path

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


class FileOperation(NamedTuple):
    """A FileOperation is a simple container class describing the operations ran by the PhotoArchiver.
    These operations have a `source_path` and a `destination_path`.
    """
    source_path: str
    destination_path: str


class PhotoArchiver:
    """Provides the logic to execute a set of file operations to archive a set of files from a list or single
    `source_directory` into a `destination_directory`. Set `copy_files` to True if you are looking to copy the files,
    false to move them.
    """

    def __init__(self,
                 source_directories: Union[str, List[str]],
                 destination_directory: str,
                 copy_files: bool = True):
        self.source_directories = source_directories if isinstance(source_directories, list) else [source_directories]
        self.destination_directory = destination_directory
        self.copy_files = copy_files

    def run(self):
        logger.info("Running PhotoArchiver")
        file_operations = self._get_file_operations()
        self._execute_file_operations(file_operations)

    def _get_file_operations(self):
        logger.info("Getting FileOperation objects")
        return [
            FileOperation(
                source_path=file_path,
                destination_path=os.path.join(self.destination_directory, get_destination_path(file_path))
            )
            for file_path in tqdm(self._get_file_paths())
        ]

    def _execute_file_operations(self, file_operations: List[FileOperation]):
        logger.info("Executing file operations (move/copy)")
        for source_path, destination_path in tqdm(file_operations):
            os.makedirs(os.path.dirname(destination_path), exist_ok=True)

            destination_path = get_deduplicated_destination_path(destination_path)

            if self.copy_files:
                shutil.copy2(source_path, destination_path)
            else:
                shutil.move(source_path, destination_path)

    def _get_file_paths(self):
        return [os.path.join(dp, f)
                for source_directory in self.source_directories
                for dp, dn, filenames in os.walk(source_directory)
                for f in filenames]



import sys
import argparse

from photoarchiver.photoarchiver import PhotoArchiver


def main():
    parser = argparse.ArgumentParser(formatter_class=argparse.RawTextHelpFormatter,
                                     description='Sort files by date using file metadata.')
    parser.add_argument('--source-directory-list', nargs='+',
                        help='Source directories to get files recursively from.',
                        required=True)
    parser.add_argument('--destination-directory', type=str, help='Destination directory', required=True)
    parser.add_argument('--move-files', action='store_true',
                        help='If True, files are copied. Otherwise, files are moved.',
                        default=False)

    args = parser.parse_args()

    photo_archiver = PhotoArchiver(source_directories=args.source_directory_list,
                                   destination_directory=args.destination_directory,
                                   copy_files=not args.move_files)
    photo_archiver.run()


if __name__ == '__main__':
    sys.exit(main())



import os

from photoarchiver.helpers.exif import get_file_metadata


def get_file_extension(file_path: str) -> str:
    return os.path.splitext(file_path)[-1].lower()


def get_destination_path(file_path) -> str:
    file_metadata = get_file_metadata(file_path)
    file_extension = get_file_extension(file_path)
    file_directory = f"{file_metadata['creation_time']:%Y/%-m - %B - Week %V}"

    # create new file name
    time_string = f"{file_metadata['creation_time']:%Y-%b-%d %H-%M-%S}"
    location_string = "-".join(filter(None, [file_metadata.get("location").get(k)
                                             for k in ["cc", "name"] if file_metadata.get("location")]))

    new_file_name = "-".join(filter(None, [time_string, location_string])) + file_extension
    return os.path.join(file_directory, new_file_name)


def get_deduplicated_destination_path(destionation_path) -> str:
    counter = 1
    deduplicated_destination_path = destionation_path
    while os.path.exists(deduplicated_destination_path):
        path_parts = destionation_path.rsplit(".")
        path_parts[0] = f"{path_parts[0]}_{counter}"
        counter += 1
        deduplicated_destination_path = "".join(path_parts)

    return deduplicated_destination_path



from setuptools import setup
from pathlib import Path

this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(name='photoarchiver',
      version='0.2',
      description='Archive your photos and files using file timestamps and geolocation.',
      url='http://github.com/baigo/photoarchiver',
      author='Martín Baigorria Alonso',
      author_email='martinbaigorria@gmail.com',
      license='MIT',
      packages=['photoarchiver', 'photoarchiver.helpers'],
      install_requires=[
          'exifread',
          'reverse_geocoder',
          'tqdm'
      ],
      zip_safe=False,
      entry_points={
          'console_scripts': [
              'photoarchiver = photoarchiver.__main__:main',
          ]
      },
      long_description=long_description,
      long_description_content_type='text/markdown')



from typing import Tuple, Dict

import exifread
from datetime import datetime
import os
import reverse_geocoder as rg

Coordinates = Tuple[float, float]
_LOCATION_CACHE = {}


# based on: https://gist.github.com/snakeye/fdc372dbf11370fe29eb

def get_exif_tags(file_path: str) -> Dict:
    with open(file_path, 'rb') as f:
        exif_tags = exifread.process_file(f)

    return exif_tags


def get_file_metadata(file_path: str) -> Dict:
    exif_tags = get_exif_tags(file_path)

    creation_time = get_exif_creation(exif_tags)
    location = get_exif_location(exif_tags)

    if creation_time is None:
        creation_time = get_creation_time(file_path)

    return {
        "creation_time": creation_time,
        "location": location
    }


def get_creation_time(file_path: str) -> datetime:
    return datetime.utcfromtimestamp(os.path.getctime(file_path))


def _convert_to_degress(value: exifread.utils.Ratio) -> float:
    """
    Helper function to convert the GPS coordinates stored in the EXIF to degrees in float format
    """
    d = float(value.values[0].num) / float(value.values[0].den)
    m = float(value.values[1].num) / float(value.values[1].den)
    s = float(value.values[2].num) / float(value.values[2].den)

    return d + (m / 60.0) + (s / 3600.0)


def get_exif_location_coordinates(exif_data: Dict) -> Coordinates:
    """
    Returns the latitude and longitude, if available, from the provided exif_data (obtained through get_exif_data above)
    """
    lat = None
    lon = None

    gps_latitude = exif_data.get('GPS GPSLatitude')
    gps_latitude_ref = exif_data.get('GPS GPSLatitudeRef')
    gps_longitude = exif_data.get('GPS GPSLongitude')
    gps_longitude_ref = exif_data.get('GPS GPSLongitudeRef')

    if gps_latitude and gps_latitude_ref and gps_longitude and gps_longitude_ref:
        lat = _convert_to_degress(gps_latitude)
        if gps_latitude_ref.values[0] != 'N':
            lat = 0 - lat

        lon = _convert_to_degress(gps_longitude)
        if gps_longitude_ref.values[0] != 'E':
            lon = 0 - lon

    return lat, lon


def get_exif_location(exif_data: Dict) -> Dict:
    location_coordinates = get_exif_location_coordinates(exif_data)

    if location_coordinates[0] is None:
        return None

    rounded_coordinates = round_coordinates(location_coordinates)

    if rounded_coordinates in _LOCATION_CACHE:
        return _LOCATION_CACHE[rounded_coordinates]

    location = rg.search(location_coordinates)[0]
    _LOCATION_CACHE[rounded_coordinates] = location

    return location


def get_exif_creation(exif_data: Dict) -> datetime:
    creation_time = None

    if 'EXIF DateTimeOriginal' in exif_data:
        creation_time = datetime.strptime(str(exif_data['EXIF DateTimeOriginal']), '%Y:%m:%d %H:%M:%S')

    return creation_time


def round_coordinates(location_coordinates: Coordinates) -> Coordinates:
    return tuple(round(c, 2) for c in location_coordinates)
